@@ -5,6 +5,7 @@
 #include <ctype.h>
 #include <limits.h>
 #include <sys/stat.h>
+#include <unistd.h>
 
 #include "imapfilter.h"
 #include "session.h"
@@ -46,11 +47,17 @@ get_cert(session *ssn)
 
 	switch (check_cert(cert, md, &mdlen)) {
 	case 0:
+		if (isatty(STDIN_FILENO) == 0)
+			fatal(ERROR_CERTIFICATE, "%s\n",
+			    "can't accept certificate in non-interactive mode");
 		print_cert(cert, md, &mdlen);
 		if (write_cert(cert) == -1)
 			goto fail;
 		break;
 	case -1:
+		if (isatty(STDIN_FILENO) == 0)
+			fatal(ERROR_CERTIFICATE, "%s\n",
+			    "certificate mismatch in non-interactive mode");
 		print_cert(cert, md, &mdlen);
 		if (mismatch_cert() == -1)
 			goto fail;
@@ -167,7 +174,8 @@ write_cert(X509 *cert)
 	do {
 		printf("(R)eject, accept (t)emporarily or "
 		    "accept (p)ermanently? ");
-		fgets(buf, sizeof(buf), stdin);
+		if (fgets(buf, sizeof(buf), stdin) == NULL)
+			return -1;
 		c = tolower((int)(*buf));
 	} while (c != 'r' && c != 't' && c != 'p');
 
@@ -214,7 +222,8 @@ mismatch_cert(void)
 	do {
 		printf("ATTENTION: SSL/TLS certificate fingerprint mismatch.\n"
 		    "Proceed with the connection (y/n)? ");
-		fgets(buf, sizeof(buf), stdin);
+		if (fgets(buf, sizeof(buf), stdin) == NULL)
+			return -1;
 		c = tolower((int)(*buf));
 	} while (c != 'y' && c != 'n');
 
