@@ -24,6 +24,8 @@ static int ifcore_fetchsize(lua_State *lua);
 static int ifcore_fetchheader(lua_State *lua);
 static int ifcore_fetchtext(lua_State *lua);
 static int ifcore_fetchfields(lua_State *lua);
+static int ifcore_fetchstructure(lua_State *lua);
+static int ifcore_fetchpart(lua_State *lua);
 static int ifcore_store(lua_State *lua);
 static int ifcore_copy(lua_State *lua);
 static int ifcore_append(lua_State *lua);
@@ -32,6 +34,7 @@ static int ifcore_delete(lua_State *lua);
 static int ifcore_rename(lua_State *lua);
 static int ifcore_subscribe(lua_State *lua);
 static int ifcore_unsubscribe(lua_State *lua);
+static int ifcore_idle(lua_State *lua);
 
 
 /* Lua imapfilter core library functions. */
@@ -68,8 +71,11 @@ static const luaL_reg ifcorelib[] = {
 	{ "fetchbody", ifcore_fetchtext }, 
 
 	{ "fetchfields", ifcore_fetchfields },
+	{ "fetchstructure", ifcore_fetchstructure },
+	{ "fetchpart", ifcore_fetchpart },
 	{ "store", ifcore_store },
 	{ "copy", ifcore_copy },
+	{ "idle", ifcore_idle },
 	{ NULL, NULL }
 };
 
@@ -180,7 +186,7 @@ ifcore_status(lua_State *lua)
 	int r;
 	unsigned int exists, recent, unseen;
 
-	exists = recent = unseen = 0;
+	exists = recent = unseen = -1;
 
 	if (lua_gettop(lua) != 2)
 		luaL_error(lua, "wrong number of arguments");
@@ -618,6 +624,47 @@ ifcore_fetchsize(lua_State *lua)
 }
 
 
+/*
+ * Core function to fetch message body structure.
+ */
+static int
+ifcore_fetchstructure(lua_State *lua)
+{
+	const char *s, *u, *p;
+	int r;
+	char *structure;
+
+	structure = NULL;
+
+	if (lua_gettop(lua) != 2)
+		luaL_error(lua, "wrong number of arguments");
+
+	luaL_checktype(lua, 1, LUA_TTABLE);
+	luaL_checktype(lua, 2, LUA_TSTRING);
+
+	lua_pushvalue(lua, 1);
+	if (!(s = get_table_string("server")))
+		luaL_error(lua, "no mail server specified");
+	if (!(u = get_table_string("username")))
+		luaL_error(lua, "no username specified");
+	p = DISCOVER_PORT(get_table_string("port"), get_table_string("ssl"));
+	lua_pop(lua, 1);
+
+	r = request_fetchstructure(s, p, u, lua_tostring(lua, 2), &structure);
+
+	lua_pop(lua, 3);
+
+	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));
+
+	if (!structure)
+		return 1;
+
+	lua_pushstring(lua, structure);
+
+	return 2;
+}
+
+
 /*
  * Core function to fetch message header.
  */
@@ -749,6 +796,51 @@ ifcore_fetchfields(lua_State *lua)
 }
 
 
+/*
+ * Core function to fetch message specific part.
+ */
+static int
+ifcore_fetchpart(lua_State *lua)
+{
+	const char *s, *u, *p;
+	int r;
+	char *part;
+	size_t len;
+
+	part = NULL;
+	len = 0;
+
+	if (lua_gettop(lua) != 3)
+		luaL_error(lua, "wrong number of arguments");
+
+	luaL_checktype(lua, 1, LUA_TTABLE);
+	luaL_checktype(lua, 2, LUA_TSTRING);
+	luaL_checktype(lua, 3, LUA_TSTRING);
+
+	lua_pushvalue(lua, 1);
+	if (!(s = get_table_string("server")))
+		luaL_error(lua, "no mail server specified");
+	if (!(u = get_table_string("username")))
+		luaL_error(lua, "no username specified");
+	p = DISCOVER_PORT(get_table_string("port"), get_table_string("ssl"));
+	lua_pop(lua, 1);
+
+	r = request_fetchpart(s, p, u, lua_tostring(lua, 2),
+	    lua_tostring(lua, 3), &part, &len);
+
+	lua_pop(lua, 3);
+
+	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));
+
+	if (!part)
+		return 1;
+
+	lua_pushlstring(lua, part, len);
+
+	return 2;
+}
+
+
 /*
  * Core function to change message flags.
  */
@@ -1030,6 +1122,36 @@ ifcore_unsubscribe(lua_State *lua)
 }
 
 
+/*
+ * Core function to go to idle state.
+ */
+static int
+ifcore_idle(lua_State *lua)
+{
+	const char *s, *u, *p;
+	int r;
+
+	if (lua_gettop(lua) != 1)
+		luaL_error(lua, "wrong number of arguments");
+
+	luaL_checktype(lua, 1, LUA_TTABLE);
+
+	if (!(s = get_table_string("server")))
+		luaL_error(lua, "no mail server specified");
+	if (!(u = get_table_string("username")))
+		luaL_error(lua, "no username specified");
+	p = DISCOVER_PORT(get_table_string("port"), get_table_string("ssl"));
+
+	r = request_idle(s, p, u);
+
+	lua_pop(lua, 1);
+
+	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));
+
+	return 1;
+}
+
+
 /*
  * Open imapfilter core library.
  */
@@ -1037,7 +1159,7 @@ LUALIB_API int
 luaopen_ifcore(lua_State *lua)
 {
 
-	luaL_openlib(lua, "ifcore", ifcorelib, 0);
+	luaL_register(lua, "ifcore", ifcorelib);
 
 	return 1;
 }
