@@ -21,6 +21,7 @@
 #define ERROR_CONFIG			2
 #define ERROR_MEMALLOC			3
 #define ERROR_PATHNAME			4
+#define ERROR_CERTIFICATE		5
 
 /* IMAP protocol supported by the server. */
 #define PROTOCOL_NONE			0
@@ -33,17 +34,21 @@
 #define CAPABILITY_CRAMMD5		0x02
 #define CAPABILITY_STARTTLS		0x04
 #define CAPABILITY_CHILDREN		0x08
+#define CAPABILITY_IDLE			0x10
 
 /* Status responses and response codes. */
 #define STATUS_RESPONSE_NONE		0
 #define STATUS_RESPONSE_OK		1
 #define STATUS_RESPONSE_NO		2
 #define STATUS_RESPONSE_BAD		3
-#define STATUS_RESPONSE_BYE		4
-#define STATUS_RESPONSE_PREAUTH		5
-#define STATUS_RESPONSE_READONLY	6
-#define STATUS_RESPONSE_TRYCREATE	7
-#define STATUS_RESPONSE_CONTINUE	8
+#define STATUS_RESPONSE_UNTAGGED	4
+#define STATUS_RESPONSE_CONTINUE	5
+#define STATUS_RESPONSE_BYE		6
+#define STATUS_RESPONSE_PREAUTH		7
+#define STATUS_RESPONSE_READONLY	8
+#define STATUS_RESPONSE_TRYCREATE	9
+#define STATUS_RESPONSE_TIMEOUT		10
+
 
 /* Initial buffer size for input, output and namespace buffers. */
 #define INPUT_BUF			4096
@@ -124,6 +129,8 @@ int imap_store(session *ssn, const char *mesg, const char *mode,
     const char *flags);
 int imap_copy(session *ssn, const char *mesg, const char *mbox);
 int imap_namespace(session *ssn);
+int imap_idle(session *ssn);
+int imap_done(session *ssn);
 
 /*	log.c		*/
 void verbose(const char *info,...);
@@ -199,12 +206,16 @@ int request_fetchdate(const char *server, const char *port, const char *user,
     const char *mesg, char **date);
 int request_fetchsize(const char *server, const char *port, const char *user,
     const char *mesg, char **size);
+int request_fetchstructure(const char *server, const char *port,
+    const char *user, const char *mesg, char **structure);
 int request_fetchheader(const char *server, const char *port, const char *user,
     const char *mesg, char **header, size_t *len);
 int request_fetchtext(const char *server, const char *port, const char *user,
     const char *mesg, char **text, size_t *len);
 int request_fetchfields(const char *server, const char *port, const char *user,
     const char *mesg, const char *headerfields, char **fields, size_t *len);
+int request_fetchpart(const char *server, const char *port, const char *user,
+    const char *mesg, const char *bodypart, char **part, size_t *len);
 int request_store(const char *server, const char *port, const char *user,
     const char *mesg, const char *mode, const char *flags);
 int request_copy(const char *server, const char *port, const char *user,
@@ -222,6 +233,7 @@ int request_subscribe(const char *server, const char *port, const char *user,
     const char *mbox);
 int request_unsubscribe(const char *server, const char *port, const char *user,
     const char *mbox);
+int request_idle(const char *server, const char *port, const char *user);
 
 
 /*	response.c	*/
@@ -242,7 +254,9 @@ int response_fetchfast(session *ssn, int tag, char **flags, char **date,
 int response_fetchflags(session *ssn, int tag, char **flags);
 int response_fetchdate(session *ssn, int tag, char **date);
 int response_fetchsize(session *ssn, int tag, char **size);
+int response_fetchstructure(session *ssn, int tag, char **structure);
 int response_fetchbody(session *ssn, int tag, char **body, size_t *len);
+int response_idle(session *ssn, int tag);
 
 /*	signal.c	*/
 void catch_signals(void);
@@ -252,7 +266,8 @@ void release_signals(void);
 int open_connection(session *ssn, const char *server, const char *port,
     const char *protocol);
 int close_connection(session *ssn);
-ssize_t socket_read(session *ssn, char *buf, size_t len);
+ssize_t socket_read(session *ssn, char *buf, size_t len, long timeout,
+    int timeoutfail);
 ssize_t socket_write(session *ssn, const char *buf, size_t len);
 #ifndef NO_SSLTLS
 int open_secure_connection(session *ssn, const char *server, const char *port,
