@@ -9,6 +9,8 @@ setmetatable(Mailbox, Mailbox._mt)
 Mailbox._mt.__call = function (self, account, mailbox)
     local object = {}
 
+    object._type = 'mailbox'
+
     for key, value in pairs(Mailbox) do
         if (type(value) == 'function') then
             object[key] = value
@@ -16,8 +18,8 @@ Mailbox._mt.__call = function (self, account, mailbox)
     end
 
     object._mt = {}
-    setmetatable(object, object._mt)
     object._mt.__index = object._attach_message
+    setmetatable(object, object._mt)
 
     object._account = account
     object._mailbox = mailbox
@@ -27,7 +29,7 @@ end
 
 
 function Mailbox._attach_message(self, uid)
-    self[uid] = Message(uid)
+    self[uid] = Message(self._account, self, uid)
     return self[uid]
 end
 
@@ -69,11 +71,15 @@ function Mailbox._send_query(self, criteria, charset)
     end
 
     if (charset == nil) then
-        charset = ''
+        if (type(options) == 'table' and type(options.charset) == 'string') then
+            charset = options.charset
+        else
+            charset = ''
+        end
     end
 
     if (self._account._login_user(self._account) ~= true) then
-        return
+        return {}
     end
 
     if (self._cached_select(self, self._account._imap, self._mailbox) ~=
@@ -93,7 +99,7 @@ function Mailbox._send_query(self, criteria, charset)
 
     local t = {}
     for n in string.gmatch(results, '%d+') do
-        table.insert(t, tonumber(n))
+        table.insert(t, { self, tonumber(n) })
     end
 
     return t
@@ -101,7 +107,7 @@ end
 
 
 function Mailbox._flag_messages(self, mode, flags, messages)
-    if (#messages == 0) then
+    if (not messages or #messages == 0) then
         return
     end
 
@@ -143,7 +149,7 @@ end
 
 
 function Mailbox._copy_messages(self, dest, messages)
-    if (#messages == 0) then
+    if (not messages or #messages == 0) then
         return
     end
 
@@ -176,7 +182,7 @@ function Mailbox._copy_messages(self, dest, messages)
         end
     else
         local fast = self._fetch_fast(self, messages)
-        local msgs = self._fetch_message(self, messages)
+        local mesgs = self._fetch_message(self, messages)
 
         if (ifcore.login(dest._account._imap) ~= true) then
             return
@@ -189,7 +195,7 @@ function Mailbox._copy_messages(self, dest, messages)
                 end
             end
 
-            r = ifcore.append(dest._account._imap, dest._mailbox, msgs[i],
+            r = ifcore.append(dest._account._imap, dest._mailbox, mesgs[i],
                 table.concat(fast[i]['flags'], ' '), fast[i]['date'])
         end
     end
@@ -199,7 +205,7 @@ end
 
 
 function Mailbox._fetch_fast(self, messages)
-    if (#messages == 0) then
+    if (not messages or #messages == 0) then
         return
     end
 
@@ -236,7 +242,7 @@ function Mailbox._fetch_fast(self, messages)
 end
 
 function Mailbox._fetch_flags(self, messages)
-    if (#messages == 0) then
+    if (not messages or #messages == 0) then
         return
     end
 
@@ -269,7 +275,7 @@ function Mailbox._fetch_flags(self, messages)
 end
 
 function Mailbox._fetch_date(self, messages)
-    if (#messages == 0) then
+    if (not messages or #messages == 0) then
         return
     end
 
@@ -284,9 +290,17 @@ function Mailbox._fetch_date(self, messages)
 
     local results = {}
     for _, m in ipairs(messages) do
-        local _, date = ifcore.fetchdate(self._account._imap, tostring(m))
-        if (date ~= nil) then
-            results[m] = date
+        if (type(options) == 'table' and options.cache == true and
+            self[m]._date) then
+            results[m] = self[m]._date
+        else
+            local _, date = ifcore.fetchdate(self._account._imap, tostring(m))
+            if (date ~= nil) then
+                results[m] = date
+                if (type(options) == 'table' and options.cache == true) then
+                    self[m]._date = date
+                end
+            end
         end
     end
 
@@ -298,7 +312,7 @@ function Mailbox._fetch_date(self, messages)
 end
 
 function Mailbox._fetch_size(self, messages)
-    if (#messages == 0) then
+    if (not messages or #messages == 0) then
         return
     end
 
@@ -313,9 +327,17 @@ function Mailbox._fetch_size(self, messages)
 
     local results = {}
     for _, m in ipairs(messages) do
-        local _, size = ifcore.fetchsize(self._account._imap, tostring(m))
-        if (size ~= nil) then
-            results[m] = tonumber(size)
+        if (type(options) == 'table' and options.cache == true and
+            self[m]._size) then
+            results[m] = self[m]._size
+        else
+            local _, size = ifcore.fetchsize(self._account._imap, tostring(m))
+            if (size ~= nil) then
+                results[m] = tonumber(size)
+                if (type(options) == 'table' and options.cache == true) then
+                    self[m]._size = tonumber(size)
+                end
+            end
         end
     end
 
@@ -327,7 +349,7 @@ function Mailbox._fetch_size(self, messages)
 end
 
 function Mailbox._fetch_header(self, messages)
-    if (#messages == 0) then
+    if (not messages or #messages == 0) then
         return
     end
 
@@ -342,14 +364,17 @@ function Mailbox._fetch_header(self, messages)
 
     local results = {}
     for _, m in ipairs(messages) do
-        if (self[m].cached_header(self[m])) then
-            results[m] = self[m].get_header(self[m])
+        if (type(options) == 'table' and options.cache == true and
+            self[m]._header) then
+            results[m] = self[m]._header
         else
             local _, header = ifcore.fetchheader(self._account._imap,
                 tostring(m))
             if (header ~= nil) then
                 results[m] = header
-                self[m].set_header(self[m], header)
+                if (type(options) == 'table' and options.cache == true) then
+                    self[m]._header = header
+                end
             end
         end
     end
@@ -362,7 +387,7 @@ function Mailbox._fetch_header(self, messages)
 end
 
 function Mailbox._fetch_body(self, messages)
-    if (#messages == 0) then
+    if (not messages or #messages == 0) then
         return
     end
 
@@ -377,13 +402,16 @@ function Mailbox._fetch_body(self, messages)
 
     local results = {}
     for _, m in ipairs(messages) do
-        if (self[m].cached_body(self[m])) then
-            results[m] = self[m].get_body(self[m])
+        if (type(options) == 'table' and options.cache == true and
+            self[m]._body) then
+            results[m] = self[m]._body
         else
             local _, body = ifcore.fetchbody(self._account._imap, tostring(m))
             if (body ~= nil) then
                 results[m] = body
-                self[m].set_body(self[m], body)
+                if (type(options) == 'table' and options.cache == true) then
+                    self[m]._body = body
+                end
             end
         end
     end
@@ -396,7 +424,7 @@ function Mailbox._fetch_body(self, messages)
 end
 
 function Mailbox._fetch_message(self, messages)
-    if (#messages == 0) then
+    if (not messages or #messages == 0) then
         return
     end
 
@@ -405,7 +433,13 @@ function Mailbox._fetch_message(self, messages)
 
     local results = {}
     for _, m in ipairs(messages) do
-        results[m] = header[m] .. body[m]
+        if (header[m] == nil) then
+            results[m] = nil
+        elseif (body[m] == nil) then
+            results[m] = header[m]
+        else
+            results[m] = header[m] .. body[m]
+        end
     end
 
     return results
@@ -413,7 +447,7 @@ end
 
 
 function Mailbox._fetch_fields(self, fields, messages)
-    if (#messages == 0) then
+    if (not messages or #messages == 0) then
         return
     end
 
@@ -430,15 +464,58 @@ function Mailbox._fetch_fields(self, fields, messages)
     for _, m in ipairs(messages) do
         results[m] = ''
         for _, f in ipairs(fields) do
-            if (self[m].cached_field(self[m], f)) then
-                results[m] = results[m] .. self[m].get_field(self[m], f)
+            if (type(options) == 'table' and options.cache == true and
+                self[m]._fields[f]) then
+                results[m] = results[m] .. self[m]._fields[f]
             else
                 local _, field = ifcore.fetchfields(self._account._imap,
                     tostring(m), f)
                 if (field ~= nil) then
-                    field = string.gsub(field, "\r\n\r\n$", "\r\n")
+                    field = string.gsub(field, "\r\n\r\n$", "\n")
                     results[m] = results[m] .. field
-                    self[m].set_field(self[m], f, field)
+                    if (type(options) == 'table' and options.cache == true) then
+                        self[m]._fields[f] = field
+                    end
+                end
+            end
+        end
+        results[m] = string.gsub(results[m], "\n$", "")
+    end
+
+    if (type(options) == 'table' and options.close == true) then
+        self._cached_close(self, self._account._imap)
+    end
+
+    return results
+end
+
+function Mailbox._fetch_structure(self, messages)
+    if (not messages or #messages == 0) then
+        return
+    end
+
+    if (self._account._login_user(self._account) ~= true) then
+        return
+    end
+
+    if (self._cached_select(self, self._account._imap, self._mailbox) ~=
+        true) then
+        return
+    end
+
+    local results = {}
+    for _, m in ipairs(messages) do
+        if (type(options) == 'table' and options.cache == true and
+            self[m]._structure) then
+            results[m] = self[m]._structure
+        else
+            local _, structure = ifcore.fetchstructure(self._account._imap,
+                tostring(m))
+            if (structure ~= nil) then
+                local parsed = _parse_structure({ ['s'] = structure, ['i'] = 1 })
+                results[m] = parsed
+                if (type(options) == 'table' and options.cache == true) then
+                    self[m]._structure = parsed
                 end
             end
         end
@@ -451,6 +528,40 @@ function Mailbox._fetch_fields(self, fields, messages)
     return results
 end
 
+function Mailbox._fetch_parts(self, parts, message)
+    if (self._account._login_user(self._account) ~= true) then
+        return
+    end
+
+    if (self._cached_select(self, self._account._imap, self._mailbox) ~=
+        true) then
+        return
+    end
+
+    local results = {}
+    for _, part in ipairs(parts) do
+        results[part] = ''
+        if (type(options) == 'table' and options.cache == true and
+            self[message]._parts[part]) then
+            results[part] = self[message]._parts[part]
+        else
+            local _, bodypart = ifcore.fetchpart(self._account._imap,
+                tostring(message), part)
+            if (bodypart ~= nil) then
+                results[part] = bodypart
+                self[message]._parts[part] = bodypart
+            end
+        end
+    end
+
+    if (type(options) == 'table' and options.close == true) then
+        self._cached_close(self, self._account._imap)
+    end
+
+    return results
+end
+
+
 function Mailbox.check_status(self)
     if (self._account._login_user(self._account) ~= true) then
         return
@@ -482,8 +593,8 @@ end
 function Mailbox.add_flags(self, flags, messages)
     _check_required(flags, 'table')
     _check_required(messages, 'table')
-
-    local mesgs = _make_normal(messages)
+    
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._flag_messages(self, 'add', flags, mesgs)
 
@@ -500,7 +611,7 @@ function Mailbox.remove_flags(self, flags, messages)
     _check_required(flags, 'table')
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._flag_messages(self, 'remove', flags, mesgs)
 
@@ -517,7 +628,7 @@ function Mailbox.replace_flags(self, flags, messages)
     _check_required(flags, 'table')
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._flag_messages(self, 'replace', flags, mesgs)
 
@@ -534,7 +645,7 @@ end
 function Mailbox.mark_answered(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._flag_messages(self, 'add', { '\\Answered' }, mesgs)
 
@@ -551,7 +662,7 @@ end
 function Mailbox.mark_deleted(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._flag_messages(self, 'add', { '\\Deleted' }, mesgs)
 
@@ -567,7 +678,7 @@ end
 function Mailbox.mark_draft(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._flag_messages(self, 'add', { '\\Draft' }, mesgs)
 
@@ -583,7 +694,7 @@ end
 function Mailbox.mark_flagged(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._flag_messages(self, 'add', { '\\Flagged' }, mesgs)
 
@@ -599,7 +710,7 @@ end
 function Mailbox.mark_seen(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._flag_messages(self, 'add', { '\\Seen' }, mesgs)
 
@@ -615,7 +726,7 @@ end
 function Mailbox.unmark_answered(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._flag_messages(self, 'remove', { '\\Answered' }, mesgs)
 
@@ -631,7 +742,7 @@ end
 function Mailbox.unmark_deleted(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._flag_messages(self, 'remove', { '\\Deleted' }, mesgs)
 
@@ -647,7 +758,7 @@ end
 function Mailbox.unmark_draft(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._flag_messages(self, 'remove', { '\\Draft' }, mesgs)
 
@@ -663,7 +774,7 @@ end
 function Mailbox.unmark_flagged(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._flag_messages(self, 'remove', { '\\Flagged' }, mesgs)
 
@@ -679,7 +790,7 @@ end
 function Mailbox.unmark_seen(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._flag_messages(self, 'remove', { '\\Seen' }, mesgs)
 
@@ -696,7 +807,7 @@ end
 function Mailbox.delete_messages(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._flag_messages(self, 'add', { '\\Deleted' }, mesgs)
 
@@ -714,7 +825,7 @@ function Mailbox.copy_messages(self, dest, messages)
     _check_required(dest, 'table')
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local r = self._copy_messages(self, dest, mesgs)
 
@@ -735,7 +846,7 @@ function Mailbox.move_messages(self, dest, messages)
     _check_required(dest, 'table')
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    local mesgs = _extract_messages(self, messages)
 
     local rc = self._copy_messages(self, dest, mesgs)
     local rf = false
@@ -757,69 +868,60 @@ function Mailbox.move_messages(self, dest, messages)
 end
 
 
-function Mailbox.fetch_fast(self, messages)
-    _check_required(messages, 'table')
-
-    local mesgs = _make_normal(messages)
-
-    return self._fetch_fast(self, mesgs)
-end
-
 function Mailbox.fetch_flags(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
-
-    return self._fetch_flags(self, mesgs)
+    return self._fetch_flags(self, _extract_messages(self, messages))
 end
 
 function Mailbox.fetch_date(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
-
-    return self._fetch_date(self, mesgs)
+    return self._fetch_date(self, _extract_messages(self, messages))
 end
 
 function Mailbox.fetch_size(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
-
-    return self._fetch_size(self, mesgs)
+    return self._fetch_size(self, _extract_messages(self, messages))
 end
 
 function Mailbox.fetch_header(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
-
-    return self._fetch_header(self, mesgs)
+    return self._fetch_header(self, _extract_messages(self, messages))
 end
 
 function Mailbox.fetch_body(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
-
-    return self._fetch_body(self, mesgs)
+    return self._fetch_body(self, _extract_messages(self, messages))
 end
 
 function Mailbox.fetch_message(self, messages)
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
-
-    return self._fetch_message(self, mesgs)
+    return self._fetch_message(self, _extract_messages(self, messages))
 end
 
 function Mailbox.fetch_fields(self, fields, messages)
     _check_required(fields, 'table')
     _check_required(messages, 'table')
 
-    local mesgs = _make_normal(messages)
+    return self._fetch_fields(self, fields, _extract_messages(self, messages))
+end
 
-    return self._fetch_fields(self, fields, mesgs)
+function Mailbox.fetch_structure(self, messages)
+    _check_required(messages, 'table')
+
+    return self._fetch_structure(self, _extract_messages(self, messages))
+end
+
+function Mailbox.fetch_parts(self, parts, message)
+    _check_required(parts, 'table')
+    _check_required(message, 'number')
+
+    return self._fetch_parts(self, parts, message)
 end
 
 
@@ -1001,121 +1103,141 @@ function Mailbox.contain_message(self, string)
 end
 
 
-function Mailbox.match_field(self, field, pattern)
+function Mailbox.match_field(self, field, pattern, messages)
     _check_required(field, 'string')
     _check_required(pattern, 'string')
 
-    local messages = self._send_query(self)
-    local mesgs = _make_normal(messages)
+    if not messages then
+        messages = self._send_query(self)
+    end
+    local mesgs = _extract_messages(self, messages)
     local fields = self._fetch_fields(self, { field }, mesgs)
 
     if (#mesgs == 0 or fields == nil) then
-        return
+        return Set({})
     end
  
     local results = {}
-    for m, f in ipairs(fields) do
+    for m, f in pairs(fields) do
         if (regex_search(pattern, f)) then
-            table.insert(results, m)
+            table.insert(results, {self, m})
         end
     end
 
     return Set(results)
 end
 
-function Mailbox.match_bcc(self, pattern)
+function Mailbox.match_bcc(self, pattern, messages)
     _check_required(pattern, 'string')
 
-    return self.match_field(self, 'Bcc', pattern)
+    return self.match_field(self, 'Bcc', pattern, messages)
 end
 
-function Mailbox.match_cc(self, pattern)
+function Mailbox.match_cc(self, pattern, messages)
     _check_required(pattern, 'string')
 
-    return self.match_field(self, 'Cc', pattern)
+    return self.match_field(self, 'Cc', pattern, messages)
 end
 
-function Mailbox.match_from(self, pattern)
+function Mailbox.match_from(self, pattern, messages)
     _check_required(pattern, 'string')
 
-    return self.match_field(self, 'From', pattern)
+    return self.match_field(self, 'From', pattern, messages)
 end
 
-function Mailbox.match_subject(self, pattern)
+function Mailbox.match_subject(self, pattern, messages)
     _check_required(pattern, 'string')
 
-    return self.match_field(self, 'Subject', pattern)
+    return self.match_field(self, 'Subject', pattern, messages)
 end
 
-function Mailbox.match_to(self, pattern)
+function Mailbox.match_to(self, pattern, messages)
     _check_required(pattern, 'string')
 
-    return self.match_field(self, 'To', pattern)
+    return self.match_field(self, 'To', pattern, messages)
 end
 
-function Mailbox.match_header(self, pattern)
+function Mailbox.match_header(self, pattern, messages)
     _check_required(pattern, 'string')
 
-    local messages = self._send_query(self)
-    local mesgs = _make_normal(messages)
+    if not messages then
+        messages = self._send_query(self)
+    end
+    local mesgs = _extract_messages(self, messages)
     local header = self._fetch_header(self, mesgs)
 
     if (#mesgs == 0 or header == nil) then
-        return
+        return Set({})
     end
 
     local results = {}
-    for m, h in ipairs(header) do
+    for m, h in pairs(header) do
         if (regex_search(pattern, h)) then
-            table.insert(results, m)
+            table.insert(results, {self, m})
         end
     end
 
     return Set(results)
 end
 
-function Mailbox.match_body(self, pattern)
+function Mailbox.match_body(self, pattern, messages)
     _check_required(pattern, 'string')
 
-    local messages = self._send_query(self)
-    local mesgs = _make_normal(messages)
+    if not messages then
+        messages = self._send_query(self)
+    end
+    local mesgs = _extract_messages(self, messages)
     local body = self._fetch_body(self, mesgs)
 
     if (#mesgs == 0 or body == nil) then
-        return
+        return Set({})
     end
 
     local results = {}
-    for m, b in ipairs(body) do
+    for m, b in pairs(body) do
         if (regex_search(pattern, b)) then
-            table.insert(results, m)
+            table.insert(results, {self, m})
         end
     end
 
     return Set(results)
 end
 
-function Mailbox.match_message(self, pattern)
+function Mailbox.match_message(self, pattern, messages)
     _check_required(pattern, 'string')
 
-    local messages = self._send_query(self)
-    local mesgs = _make_normal(messages)
+    if not messages then
+        messages = self._send_query(self)
+    end
+    local mesgs = _extract_messages(self, messages)
     local full = self._fetch_message(self, mesgs)
 
     if (#mesgs == 0 or full == nil) then
-        return
+        return Set({})
     end
 
     local results = {}
-    for m, f in ipairs(full) do
+    for m, f in pairs(full) do
         if (regex_search(pattern, f)) then
-            table.insert(results, m)
+            table.insert(results, {self, m})
         end
     end
 
     return Set(results)
 end
 
+function Mailbox.enter_idle(self)
+    if (self._account._login_user(self._account) ~= true) then
+        return false
+    end
+
+    if (self._cached_select(self, self._account._imap, self._mailbox) ~=
+        true) then
+        return false
+    end
+   
+    return ifcore.idle(self._account._imap)
+end
 
 Mailbox._mt.__index = function () end
 Mailbox._mt.__newindex = function () end
