@@ -163,8 +163,6 @@ request_status(const char *server, const char *port, const char *user,
 
 	m = apply_namespace(mbox, s->ns.prefix, s->ns.delim);
 
-	*exists = *recent = *unseen = -1;
-
 	if (s->protocol == PROTOCOL_IMAP4REV1) {
 		t = imap_status(s, m, "MESSAGES RECENT UNSEEN");
 		if ((r = response_status(s, t, exists, recent, unseen)) == -1)
@@ -446,6 +444,32 @@ fail:
 }
 
 
+/*
+ * Fetch the body structure, ie. BODYSTRUCTURE, of the messages.
+ */
+int
+request_fetchstructure(const char *server, const char *port, const char *user,
+    const char *mesg, char **structure)
+{
+	int t, r;
+	session *s;
+
+	if (!(s = session_find(server, port, user)))
+		return -1;
+
+	t = imap_fetch(s, mesg, "BODYSTRUCTURE");
+	if ((r = response_fetchstructure(s, t, structure)) == -1)
+		goto fail;
+
+	return r;
+fail:
+	close_connection(s);
+	session_destroy(s);
+
+	return -1;
+}
+
+
 /*
  * Fetch the header, ie. BODY[HEADER], of the messages.
  */
@@ -499,8 +523,8 @@ fail:
 
 
 /*
- * Fetch the specified header fields, ie. BODY[HEADER.FIELDS ()], of the
- * messages.
+ * Fetch the specified header fields, ie. BODY[HEADER.FIELDS (<fields>)], of
+ * the messages.
  */
 int
 request_fetchfields(const char *server, const char *port, const char *user,
@@ -532,6 +556,40 @@ fail:
 }
 
 
+/*
+ * Fetch the specified message part, ie. BODY[<part>], of the
+ * messages.
+ */
+int
+request_fetchpart(const char *server, const char *port, const char *user,
+    const char *mesg, const char *part, char **bodypart, size_t *len)
+{
+	int t, r, n;
+	session *s;
+	char *f;
+
+	n = strlen("BODY.PEEK[]") + strlen(part) + 1;
+	f = (char *)xmalloc(n * sizeof(char));
+	snprintf(f, n, "%s%s%s", "BODY.PEEK[", part, "]");
+
+	if (!(s = session_find(server, port, user)))
+		return -1;
+
+	t = imap_fetch(s, mesg, f);
+	if ((r = response_fetchbody(s, t, bodypart, len)) == -1)
+		goto fail;
+
+	xfree(f);
+
+	return r;
+fail:
+	close_connection(s);
+	session_destroy(s);
+
+	return -1;
+}
+
+
 /*
  * Add, remove or replace the specified flags of the messages.
  */
@@ -795,6 +853,47 @@ fail:
 }
 
 
+int
+request_idle(const char *server, const char *port, const char *user)
+{
+	int t, rg, ri;
+	session *s;
+
+	if (!(s = session_find(server, port, user)))
+		return -1;
+
+
+	if (!(s->capabilities & CAPABILITY_IDLE))
+		return -1;
+
+	do {
+		ri = 0;
+
+		t = imap_idle(s);
+
+		if ((rg = response_generic(s, t)) == -1)
+			goto fail;
+
+		if (rg == STATUS_RESPONSE_CONTINUE) {
+			if ((ri = response_idle(s, t)) == -1)
+				goto fail;
+
+			imap_done(s);
+
+			if ((rg = response_generic(s, t)) == -1)
+				goto fail;
+		}
+	} while (ri == STATUS_RESPONSE_TIMEOUT);
+
+	return rg;
+fail:
+	close_connection(s);
+	session_destroy(s);
+
+	return -1;
+}
+
+
 /*
  * Auxiliary function to create a mailbox.
  */
