@@ -795,7 +795,8 @@ response_idle(session *ssn, int tag)
 	do {
 		buffer_reset(&ibuf);
 
-		switch (receive_response(ssn, ibuf.data, 29 * 60, 0)) {
+		switch (receive_response(ssn, ibuf.data,
+		    get_option_number("keepalive") * 60, 0)) {
 		case -1:
 			return -1;
 			break; /* NOTREACHED */
