@@ -32,12 +32,12 @@ become_daemon(600, forever)
 
 all = myaccount.mymailbox:select_all()
 
-results = {}
+results = Set {}
 for _, mesg in ipairs(all) do
     mbox, uid = unpack(mesg)
     text = mbox[uid]:fetch_message()
     if (pipe_to('bayesian-spam-filter', text) == 1) then
-        results.insert(mesg)
+        table.insert(results, mesg)
     end
 end
 
@@ -50,7 +50,7 @@ results:delete_messages()
 
 all = myaccount.mymailbox:select_all()
 
-results = {}
+results = Set {}
 for _, mesg in ipairs(all) do
     mbox, uid = unpack(mesg)
     structure = mbox[uid]:fetch_structure()
@@ -58,7 +58,7 @@ for _, mesg in ipairs(all) do
         if partinf.type:lower() == 'text/plain' and partinf.size < 1024 then
             part = mbox[uid]:fetch_part(partid)
             if (pipe_to('bayesian-spam-filter', part) == 1) then
-                results.insert(mesg)
+                table.insert(results, mesg)
                 break
             end
         end
