@@ -10,6 +10,7 @@
 /* IMAP session. */
 typedef struct session {
 	char *server;		/* Server hostname. */
+	char *port;		/* Server port. */ 
 	char *username;		/* User name. */
 	int socket;		/* Socket. */
 #ifndef NO_SSLTLS
@@ -30,7 +31,7 @@ session *session_new(void);
 void session_init(session *ssn);
 void session_destroy(session *ssn);
 void session_free(session *ssn);
-session *session_find(const char *server, const char *user);
+session *session_find(const char *server, const char *port, const char *user);
 
 
 #endif				/* SESSION_H */
