@@ -204,6 +204,7 @@ ifsys_read(lua_State *lua)
 static int
 ifsys_write(lua_State *lua)
 {
+	size_t n;
 
 	if (lua_gettop(lua) != 2)
 		luaL_error(lua, "wrong number of arguments");
@@ -211,12 +212,14 @@ ifsys_write(lua_State *lua)
 	luaL_checktype(lua, 1, LUA_TUSERDATA);
 	luaL_checktype(lua, 2, LUA_TSTRING);
 
-	fwrite(lua_tostring(lua, 2), sizeof(char), strlen(lua_tostring(lua,
+	n = fwrite(lua_tostring(lua, 2), sizeof(char), strlen(lua_tostring(lua,
 	    2)), *(FILE **) (lua_touserdata(lua, 1)));
 
 	lua_pop(lua, 2);
 
-	return 0;
+	lua_pushboolean(lua, (n != 0));
+
+	return 1;
 }
 
 
@@ -282,10 +285,11 @@ ifsys_daemon(lua_State *lua)
 	close(STDIN_FILENO);
 	close(STDOUT_FILENO);
 	close(STDERR_FILENO);
-	if (open("/dev/null", O_RDWR) != -1) {
-		dup(STDIN_FILENO);
-		dup(STDIN_FILENO);
-	}
+
+	if (open("/dev/null", O_RDWR) == -1 ||
+	    dup(STDIN_FILENO) == -1 ||
+	    dup(STDIN_FILENO) == -1)
+		fprintf(stderr, "creating session; %s\n", strerror(errno));
 
 	return 0;
 }
@@ -298,7 +302,7 @@ LUALIB_API int
 luaopen_ifsys(lua_State *lua)
 {
 
-	luaL_openlib(lua, "ifsys", ifsyslib, 0);
+	luaL_register(lua, "ifsys", ifsyslib);
 
 	return 1;
 }
